#include "WProgram.h"
#include <HardwareSerial.h>
void initBuzzer ();
void startBeep (int time);
void doBeep ();
void enableBuzzer ();
void disableBuzzer ();
void initMatrix ();
void setupMatrix ();
void clearMatrix ();
void drawMatrix ();
void printBuffer ();
void drawColumn (byte col, byte values[]);
void initLEDs ();
void clearLEDs ();
void startRead ();
void doRead ();
void startShining ();
void doShine ();
void randomShine ();
void refreshBuffer ();
void textBuffer ();
void bitStringBuffer ();
void addPixelsToBuffer (byte l[7][5], byte start, byte w);
void emptyBuffer ();
void addCharToBuffer (char l, byte start, byte w);
byte charWidth (char l);
void initMotors ();
void enableVibrator ();
void disableVibrator ();
void startVibrate (int time);
void doVibrate ();
void initI2C();
void checkI2C();
void calibrateI2C (byte direction);
void calibrateI2C();
long readValue();
void writeRegister(unsigned char r, unsigned char v);
void writeInteger(unsigned char r, unsigned int v);
unsigned char readRegister(unsigned char r);
void readRegisters(unsigned char r, unsigned int numberOfBytes, unsigned char buffer[]);
unsigned int readInteger(unsigned char r);
unsigned long readLong(unsigned char r);
void behave ();
void beNothing ();
void beTweet ();
void beFollow ();
void beMention ();
void beMessage ();
void bePhoto ();
void beVideo ();
void doSleep ();
void applyActions ();
void clearAll ();
void falseFlags();
void setup ();
void loop ();
void resetScreen ();
void resetText ();
void parseMessage ();
void BTcheck ();
void setStatus (String status);
byte lettera[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {0,0,0,1,0},
  {0,1,1,1,0},
  {1,0,0,1,0},
  {0,1,1,1,0},
  {0,0,0,0,0}
};
/**/
byte letterA[7][5] = {
  {0,0,0,0,1},
  {0,1,1,0,0},
  {0,0,0,1,0},
  {0,1,1,1,0},
  {1,0,0,1,0},
  {0,1,1,1,0},
  {0,0,0,0,0}
};
/**/
byte letterb[7][5] = {
  {0,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {1,1,1,0,0},
  {1,0,0,1,0},
  {1,1,1,0,0},
  {0,0,0,0,0}
};
byte letterc[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,1,1,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {0,1,1,1,0},
  {0,0,0,0,0}
};
byte letterC[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,1,1,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {0,1,1,1,0},
  {0,0,1,0,0}
};
byte letterd[7][5] = {
  {0,0,0,0,0},
  {0,0,0,1,0},
  {0,0,0,1,0},
  {0,1,1,1,0},
  {1,0,0,1,0},
  {0,1,1,1,0},
  {0,0,0,0,0}
};
byte lettere[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {1,1,1,0,0},
  {1,0,0,0,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
/**/
byte letterE[7][5] = {
  {0,0,0,0,1},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {1,1,1,0,0},
  {1,0,0,0,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
/**/
byte letterf[7][5] = {
  {0,0,0,0,0},
  {0,0,1,1,0},
  {0,1,0,0,0},
  {1,1,1,1,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0}
};
byte letterg[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {0,1,1,1,0},
  {0,0,0,1,0},
  {0,1,1,0,0}
};
byte letterh[7][5] = {
  {0,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {1,1,0,0,0},
  {1,0,1,0,0},
  {1,0,1,0,0},
  {0,0,0,0,0}
};
byte letteri[7][5] = {
  {0,0,0,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0},
  {1,1,0,0,0},
  {0,1,0,0,0},
  {1,1,1,0,0},
  {0,0,0,0,0}
};
/**/
byte letterI[7][5] = {
  {0,0,1,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0},
  {1,1,0,0,0},
  {0,1,0,0,0},
  {1,1,1,0,0},
  {0,0,0,0,0}
};
/**/
byte letterj[7][5] = {
  {0,0,0,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {1,0,0,0,0}
};
byte letterk[7][5] = {
  {1,0,0,0,0},
  {1,0,0,0,0},
  {1,0,1,0,0},
  {1,1,0,0,0},
  {1,0,1,0,0},
  {1,0,0,1,0},
  {0,0,0,0,0}
};
byte letterl[7][5] = {
  {1,0,0,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {1,1,1,0,0},
  {0,0,0,0,0}
};
byte letterm[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,1,1,1,0},
  {1,0,1,0,1},
  {1,0,1,0,1},
  {1,0,1,0,1},
  {0,0,0,0,0}
};
byte letterM[7][5] = {
  {1,0,0,0,1},
  {1,1,0,1,1},
  {1,0,1,0,1},
  {1,0,1,0,1},
  {1,0,1,0,1},
  {1,0,1,0,1},
  {0,0,0,0,0}
};
byte lettern[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,1,1,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,0,0,0,0}
};
byte letterN[7][5] = {
  {1,1,1,0,0},
  {0,0,0,0,0},
  {1,1,1,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,0,0,0,0}
};
byte lettero[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
/**/
byte letterO[7][5] = {
  {0,0,0,0,1},
  {0,0,0,1,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
byte letteroslash[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,1,1,0},
  {1,1,0,1,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
byte letterodieresis[7][5] = {
  {1,0,0,1,0},
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
/**/
byte letterp[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {1,1,1,0,0},
  {1,0,0,0,0}
};
byte letterq[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,1,0},
  {0,0,0,1,0}
};
byte letterr[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,1,0,0,0},
  {1,0,1,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0}
};
byte letters[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,0,0},
  {0,1,0,0,0},
  {0,0,1,0,0},
  {1,1,0,0,0},
  {0,0,0,0,0}
};
byte lettert[7][5] = {
  {0,1,0,0,0},
  {0,1,0,0,0},
  {1,1,1,1,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {0,0,1,1,0},
  {0,0,0,0,0}
};
byte letterthorn[7][5] = {
  {0,0,0,0,0},
  {1,0,0,0,0},
  {1,1,0,0,0},
  {1,0,1,0,0},
  {1,1,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0}
};
byte letteru[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,0,1},
  {0,0,0,0,0}
};
byte letterU[7][5] = {
  {0,0,0,1,0},
  {0,0,1,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,0,1},
  {0,0,0,0,0}
};
byte letterudieresis[7][5] = {
  {1,0,0,1,0},
  {0,0,0,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,0,1},
  {0,0,0,0,0}
};
byte letterv[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,0,0,0,1},
  {1,0,0,0,1},
  {0,1,0,1,0},
  {0,0,1,0,0},
  {0,0,0,0,0}
};
byte letterw[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,0,1,0,1},
  {1,0,1,0,1},
  {1,0,1,0,1},
  {0,1,0,1,0},
  {0,0,0,0,0}
};
byte letterx[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,0,0,1,0},
  {0,1,1,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {0,0,0,0,0}
};
byte lettery[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,1,0},
  {0,0,0,1,0},
  {0,1,1,0,0}
};
byte letterydieresis[7][5] = {
  {1,0,0,1,0},
  {0,0,0,0,0},
  {1,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,1,0},
  {0,0,0,1,0},
  {0,1,1,0,0}
};
byte letterz[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,1,1,1,0},
  {0,0,1,0,0},
  {0,1,0,0,0},
  {1,1,1,1,0},
  {0,0,0,0,0}
};
byte letter_[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,1,1,1,0},
  {0,0,0,0,0}
};
byte letter1[7][5] = {
  {0,0,0,0,0},
  {0,1,0,0,0},
  {1,1,0,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {1,1,1,0,0},
  {0,0,0,0,0}
};
byte letter2[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {0,0,1,0,0},
  {0,1,0,0,0},
  {1,1,1,1,0},
  {0,0,0,0,0}
};
byte letter3[7][5] = {
  {0,0,0,0,0},
  {1,1,1,1,0},
  {0,0,1,0,0},
  {0,1,1,0,0},
  {0,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,0,0}
};
byte letter4[7][5] = {
  {0,0,0,0,0},
  {0,0,0,1,0},
  {0,0,1,1,0},
  {0,1,0,1,0},
  {1,1,1,1,1},
  {0,0,0,1,0},
  {0,0,0,0,0}
};
byte letter5[7][5] = {
  {0,0,0,0,0},
  {1,1,1,1,0},
  {1,0,0,0,0},
  {1,1,1,0,0},
  {0,0,0,1,0},
  {1,0,0,1,0},
  {0,1,1,0,0}
};
byte letter6[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,0,0},
  {1,1,1,0,0},
  {1,0,0,1,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
byte letter7[7][5] = {
  {0,0,0,0,0},
  {1,1,1,1,0},
  {0,0,0,1,0},
  {0,0,1,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0}
};
byte letter8[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
byte letter9[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {0,1,1,1,0},
  {0,0,0,1,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
byte letter0[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {1,0,0,1,0},
  {1,1,1,1,0},
  {1,0,0,1,0},
  {0,1,1,0,0},
  {0,0,0,0,0}
};
byte letterMINUS[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,1,1,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0}
};
byte letterPLUS[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,0,0,0},
  {1,1,1,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0}
};
byte letterSLASH[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,1,0,0},
  {0,1,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0}
};
byte letterTIMES[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,0,1,0,0},
  {0,1,0,0,0},
  {1,0,1,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0}
};
byte letterEQUAL[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,1,1,0,0},
  {0,0,0,0,0},
  {1,1,1,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0}
};
byte letterCOLON[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0}
};
byte letterEXCLAM[7][5] = {
  {0,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0}
};
byte letterQUEST[7][5] = {
  {0,1,1,0,0},
  {1,0,0,1,0},
  {0,0,1,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0}
};
byte letterPERIOD[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0}
};
byte letterCOMMA[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,0,0,0},
  {1,0,0,0,0}
};
byte letterSEMICOLON[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0},
  {0,1,0,0,0},
  {1,0,0,0,0}
};
byte letterLPAREN[7][5] = {
  {0,0,0,0,0},
  {0,1,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0}
};
byte letterRPAREN[7][5] = {
  {0,0,0,0,0},
  {1,0,0,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {0,1,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0}
};
byte letterPIPE[7][5] = {
  {0,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {1,0,0,0,0},
  {0,0,0,0,0}
};
byte letterSPACE[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,0,0,0,0}
};
byte iconHEART[7][5] = {
  {0,0,0,0,0},
  {0,0,0,0,0},
  {0,1,0,1,0},
  {1,1,1,1,1},
  {0,1,1,1,0},
  {0,0,1,0,0},
  {0,0,0,0,0}
};
byte iconVIDEO[7][5] = {
  {0,0,0,0,0},
  {0,1,0,0,0},
  {0,1,1,0,0},
  {0,1,1,1,0},
  {0,1,1,0,0},
  {0,1,0,0,0},
  {0,0,0,0,0}
};
byte iconPHOTO[7][5] = {
  {0,0,0,0,0},
  {1,1,1,1,1},
  {1,0,0,0,1},
  {1,0,1,0,1},
  {1,0,0,0,1},
  {1,1,1,1,1},
  {0,0,0,0,0}
};
byte iconQUEST[7][5] = {
  {0,0,0,0,0},
  {0,1,1,0,0},
  {0,0,0,1,0},
  {0,0,1,0,0},
  {0,0,0,0,0},
  {0,0,1,0,0},
  {0,0,0,0,0}
};
byte iconEXCLAM[7][5] = {
  {0,0,0,0,0},
  {0,0,1,0,0},
  {0,0,1,0,0},
  {0,0,1,0,0},
  {0,0,0,0,0},
  {0,0,1,0,0},
  {0,0,0,0,0}
};


// behavior vars
int current_behavior = 0;
int current_sub_behavior = 0;

// valor desde el serial
byte serial_data;
boolean is_active = true;
String serial_message = "";
int bytes_read = 0;
int const bytes_to_read = 35;
byte serial_pixels[7][5];

// status led
int status_led_pin = 22;

// capacitive sensor
int capacitive_led_pin = 23;
long capacitive_threshold = 11070000;
unsigned long i2c_last_check = 0;
int i2c_delay = 1000;
boolean is_touched = false;
long i2c_value = 0;

// falling switches
int fwd_switch_pin = 4;
int rew_switch_pin = 5;

// buzzer
int buzzer_pin = 30;
unsigned long last_beep = 0;
int beep_length = 200;
boolean is_beeping = false;

// leds
int a_led_pin = 35;
int b_led_pin = 36;
int c_led_pin = 37;
boolean is_shining = false;
int current_led = 1;
boolean is_reading = false;
unsigned long last_read = 0;
int read_length = 500;

// vibrator
int vibrator_pin = 38;
unsigned long last_vibrate = 0;
int vibrate_length = 1000;
boolean is_vibrating = false;

// motors
int motor1pin1 = 24;
int motor1pin2 = 25;
int motor1enable_pin = 26;
int motor2pin1 = 27;
int motor2pin2 = 28;
int motor2enable_pin = 29;

// pines, filas, columnas
int matrix_pins[] = {4,5,6,7,8,9,10,11,12,13,14,15};
int cols[5];
int rows[7];
byte const matrices = 1;
byte const cols_per_matrix = 5; 

// pixeles
char my_message[] = " hello... my name is socially nervous ";
byte const buffer_cols = matrices * cols_per_matrix;
byte const buffer_rows = 7;
byte buffer[buffer_cols][buffer_rows];
int current = 0;
int bc = 0;
int cc = 0;
int ml = strlen(my_message);

// intervalos
int buffer_column = 0;
int refresh_rate = 5;
int ticker_speed = 15 * refresh_rate;
unsigned long last_tick = 0;
int bmp_delay = 2000;
unsigned long last_bmp = 0;
unsigned long last_refresh = 0;

// flags
boolean end_message = false;
int mode = 1; // 0 = string display 1 = bitstring display

// state
int state = 0;
int tmp_state = -1;
String username = "";
int activity_count = 0;
boolean state_changed = false;

// sleep state
int sleep_direction = 1;
int sleep_length = 1400;
unsigned long start_sleeping = 0;
boolean is_sleeping = false;

void initBuzzer () {
  pinMode(buzzer_pin,OUTPUT);
}

void startBeep (int time) {
  is_beeping = true;
  beep_length = time;
  last_beep = millis();
  doBeep();
}

void doBeep () {
  if (is_beeping && (millis()-last_beep < beep_length)) {
    enableBuzzer();
  } else {
    last_beep = millis();
    is_beeping = false;
    disableBuzzer();
  }
}

void enableBuzzer () {
  digitalWrite(buzzer_pin,HIGH);
}

void disableBuzzer () {
  digitalWrite(buzzer_pin,LOW);
}

void initMatrix () {
  for (int i=0;i<12;i++) {
    pinMode(matrix_pins[i],OUTPUT);
  }
  setupMatrix();
}

void setupMatrix () {
  // asociar el pin de las columnas
  cols[0] = matrix_pins[0];
  cols[1] = matrix_pins[2];
  cols[2] = matrix_pins[9];
  cols[3] = matrix_pins[6];
  cols[4] = matrix_pins[7];
  // asociar el pin de las filas
  rows[0] = matrix_pins[11];
  rows[1] = matrix_pins[10];
  rows[2] = matrix_pins[1];
  rows[3] = matrix_pins[8];
  rows[4] = matrix_pins[3];
  rows[5] = matrix_pins[4];
  rows[6] = matrix_pins[5];
}

void clearMatrix () {
  int i;
  for (i=0;i<12;i++) {
    digitalWrite(matrix_pins[i],LOW);
  }
}

void drawMatrix () {
  unsigned long m = millis();
  if ((m-last_tick)>ticker_speed) {
    last_tick = m;
    refreshBuffer();
  }
  printBuffer();
}

void printBuffer () {
  int i;
  unsigned long m = millis();
  drawColumn(buffer_column,buffer[buffer_column]);
  if (m-last_refresh>refresh_rate) {
    last_refresh = m;
    buffer_column = (buffer_column + 1 < cols_per_matrix) ? buffer_column + 1 : 0;
    clearMatrix();
  }
}

void drawColumn (byte col, byte values[]) {
  // primero prendemos toda la columna
  digitalWrite(cols[col],HIGH);
  // luego apagamos las filas que toca
  int i;
  for (i=0;i<buffer_rows;i++) {
    if (values[i]==0) digitalWrite(rows[i],HIGH);
  }
}

/*****
CORNER LEDs
******/

void initLEDs () {
  pinMode(a_led_pin,OUTPUT);
  pinMode(b_led_pin,OUTPUT);
  pinMode(c_led_pin,OUTPUT);
  //pinMode(d_led_pin,OUTPUT);
  analogWrite(a_led_pin,0);
  analogWrite(b_led_pin,0);
  analogWrite(c_led_pin,0);
  //digitalWrite(d_led_pin,LOW);
}

void clearLEDs () {
  analogWrite(a_led_pin,0);
  analogWrite(b_led_pin,0);
  analogWrite(c_led_pin,0);
}

void startRead () {
  is_reading = true;
  last_read = millis();
  doRead();
}

void doRead () {
  if (is_reading && (millis()-last_read < read_length)) {
    clearLEDs();
    if (current_led==1) {
      analogWrite(a_led_pin,1023);
    } else if (current_led==2) {
      analogWrite(b_led_pin,1023);
    } else if (current_led==3) {
      analogWrite(c_led_pin,1023);
    }
  }
 if (millis()-last_read >= read_length) {
    if (current_led==1) {
      current_led = 2;
    } else if (current_led==2) {
      current_led = 3;
    } else if (current_led==3) {
      current_led = 1;
    }
    last_read = millis();
  }
}

void startShining () {
  is_shining = true;
  doShine();
}

void doShine () {
  if (is_shining) {
    randomShine();
  }
}

void randomShine () {
  int coin;
  coin = round(random(0,4));
  analogWrite(a_led_pin,map(coin,0,4,0,1023));
  coin = round(random(0,4));
  analogWrite(b_led_pin,map(coin,0,4,0,1023));
  coin = round(random(0,4));
  analogWrite(c_led_pin,map(coin,0,4,0,1023));
}
void refreshBuffer () {
  // prepares a 5x7 grid to be drawn
  unsigned long m = millis();
  emptyBuffer();
  if (mode==0) {
    // writing text
    if (!end_message) {
      textBuffer();
    } else {
      // messeage end
      // draw bmp
      last_bmp = m;
      mode = 1;
    }
  } else {
    // drawing pixels
    if (m-last_bmp<bmp_delay) {
      // keep drawing bmp
      bitStringBuffer();
    } else {
      // times up turn to text mode
      resetText();
      mode = 0;
    }
  }
}

void textBuffer () {
  char l;
  byte w;
  // datos caracter actual
  l = my_message[current];
  w = charWidth(l);
  // chequeamos cuantos caracteres alcanzo a pintar segun el ancho que tengo
  int pixels_left = buffer_cols;
  int counter = 1;
  byte pixels_to_add;
  char temp_l;
  byte temp_w;
  pixels_to_add = w - cc;
  pixels_left -= pixels_to_add;
  if (pixels_left != buffer_cols) {
    addCharToBuffer(l,cc,pixels_to_add);
  }
  if (pixels_left>0) {
    addCharToBuffer(' ',0,1);
    pixels_left--;
  }
  // metemos el resto de chars que quepan en esos pixeles
  while (pixels_left>0) {
    // preguntar si hay un current + 1
    if ((current+counter)<ml) {
      // hay caracter pa pintar
      // le metemos un espacio si no es un espacio ya
      // el char
      temp_l = my_message[(current+counter)];
      // ancho del char
      temp_w = charWidth(temp_l);
      // miramos si los pixeles que quedan son suficientes para el char
      pixels_to_add = (pixels_left>temp_w) ? temp_w : pixels_left;
      addCharToBuffer(temp_l,0,pixels_to_add);
      pixels_left -= pixels_to_add;
      if (pixels_left>0) {
        addCharToBuffer(' ',0,1);
        pixels_left--;
      }
      counter++;
    } else {
      pixels_left = 0;
    }
  }
  if (cc<w) {
    // todavia queda pixel del char a pintar
    cc++;
  } else {
    cc = 0;
    if ((current+1)<ml) {
      current++;
    } else {
      end_message = true;
      current = 0;
    }
  }
}

void bitStringBuffer () {
  // paint bmp
  switch (state) {
    case 0:
      addPixelsToBuffer(letterSPACE,0,5);
      break;
    case 1:
      addPixelsToBuffer(iconQUEST,0,5);
      break;
    case 2:
      addPixelsToBuffer(iconHEART,0,5);
      break;
    case 3:
      addPixelsToBuffer(iconEXCLAM,0,5);
      break;
    case 4:
      addPixelsToBuffer(iconHEART,0,5);
      break;
    case 5:
      addPixelsToBuffer(iconPHOTO,0,5);
      break;
    case 6:
      addPixelsToBuffer(iconVIDEO,0,5);
      break;
    case 7:
      addPixelsToBuffer(iconHEART,0,5);
      break;
  }
}

void addPixelsToBuffer (byte l[7][5], byte start, byte w) {
  int i, j;
  for (i=0;i<buffer_rows;i++) {
    for (j=0;j<w;j++) {
      buffer[bc+j][i] = l[i][start+j];
    }
  }
  bc+=w;
}

void emptyBuffer () {
  int i, j;
  for (i=0;i<buffer_cols;i++) {
    for (j=0;j<buffer_rows;j++) {
      buffer[i][j] = 0;
    }
  }
  bc=0;
}

void addCharToBuffer (char l, byte start, byte w) {
  if (l=='a') {
    addPixelsToBuffer (lettera, start, w);
  } else if (l=='A') {
    addPixelsToBuffer (letterA, start, w);
  } else if (l=='b') {
    addPixelsToBuffer (letterb, start, w);
  } else if (l=='c') {
    addPixelsToBuffer (letterc, start, w);
  } else if (l=='C') {
    addPixelsToBuffer (letterC, start, w);
  } else if (l=='d') {
    addPixelsToBuffer (letterd, start, w);
  } else if (l=='e') {
    addPixelsToBuffer (lettere, start, w);
  } else if (l=='E') {
    addPixelsToBuffer (letterE, start, w);
  } else if (l=='f') {
    addPixelsToBuffer (letterf, start, w);
  } else if (l=='g') {
    addPixelsToBuffer (letterg, start, w);
  } else if (l=='h') {
    addPixelsToBuffer (letterh, start, w);
  } else if (l=='i') {
    addPixelsToBuffer (letteri, start, w);
  } else if (l=='I') {
    addPixelsToBuffer (letterI, start, w);
  } else if (l=='j') {
    addPixelsToBuffer (letterj, start, w);
  } else if (l=='k') {
    addPixelsToBuffer (letterk, start, w);
  } else if (l=='l') {
    addPixelsToBuffer (letterl, start, w);
  } else if (l=='m') {
    addPixelsToBuffer (letterm, start, w);
  } else if (l=='M') {
    addPixelsToBuffer (letterM, start, w);
  } else if (l=='n') {
    addPixelsToBuffer (lettern, start, w);
  } else if (l=='N') {
    addPixelsToBuffer (letterN, start, w);
  } else if (l=='o') {
    addPixelsToBuffer (lettero, start, w);
  } else if (l=='O') {
    addPixelsToBuffer (letterO, start, w);
  } else if (l=='P') {
    addPixelsToBuffer (letteroslash, start, w);
  } else if (l=='Q') {
    addPixelsToBuffer (letterodieresis, start, w);
  } else if (l=='p') {
    addPixelsToBuffer (letterp, start, w);
  } else if (l=='q') {
    addPixelsToBuffer (letterq, start, w);
  } else if (l=='r') {
    addPixelsToBuffer (letterr, start, w);
  } else if (l=='s') {
    addPixelsToBuffer (letters, start, w);
  } else if (l=='t') {
    addPixelsToBuffer (lettert, start, w);
  } else if (l=='T') {
    addPixelsToBuffer (letterthorn, start, w);
  } else if (l=='u') {
    addPixelsToBuffer (letteru, start, w);
  } else if (l=='U') {
    addPixelsToBuffer (letterU, start, w);
  } else if (l=='V') {
    addPixelsToBuffer (letterudieresis, start, w);
  } else if (l=='v') {
    addPixelsToBuffer (letterv, start, w);
  } else if (l=='w') {
    addPixelsToBuffer (letterw, start, w);
  } else if (l=='x') {
    addPixelsToBuffer (letterx, start, w);
  } else if (l=='y') {
    addPixelsToBuffer (lettery, start, w);
  } else if (l=='Y') {
    addPixelsToBuffer (letterydieresis, start, w);
  } else if (l=='z') {
    addPixelsToBuffer (letterz, start, w);
  } else if (l=='_') {
    addPixelsToBuffer (letter_, start, w);
  } else if (l=='1') {
    addPixelsToBuffer (letter1, start, w);
  } else if (l=='2') {
    addPixelsToBuffer (letter2, start, w);
  } else if (l=='3') {
    addPixelsToBuffer (letter3, start, w);
  } else if (l=='4') {
    addPixelsToBuffer (letter4, start, w);
  } else if (l=='5') {
    addPixelsToBuffer (letter5, start, w);
  } else if (l=='6') {
    addPixelsToBuffer (letter6, start, w);
  } else if (l=='7') {
    addPixelsToBuffer (letter7, start, w);
  } else if (l=='8') {
    addPixelsToBuffer (letter8, start, w);
  } else if (l=='9') {
    addPixelsToBuffer (letter9, start, w);
  } else if (l=='0') {
    addPixelsToBuffer (letter0, start, w);
  } else if (l=='-') {
    addPixelsToBuffer (letterMINUS, start, w);
  } else if (l=='+') {
    addPixelsToBuffer (letterPLUS, start, w);
  } else if (l=='/') {
    addPixelsToBuffer (letterSLASH, start, w);
  } else if (l=='*') {
    addPixelsToBuffer (letterTIMES, start, w);
  } else if (l=='=') {
    addPixelsToBuffer (letterEQUAL, start, w);
  } else if (l==':') {
    addPixelsToBuffer (letterCOLON, start, w);
  } else if (l=='.') {
    addPixelsToBuffer (letterPERIOD, start, w);
  } else if (l==',') {
    addPixelsToBuffer (letterCOMMA, start, w);
  } else if (l==';') {
    addPixelsToBuffer (letterSEMICOLON, start, w);
  } else if (l=='|') {
    addPixelsToBuffer (letterPIPE, start, w);
  } else if (l=='!') {
    addPixelsToBuffer (letterEXCLAM, start, w);
  } else if (l=='?') {
    addPixelsToBuffer (letterQUEST, start, w);
  } else if (l=='(') {
    addPixelsToBuffer (letterLPAREN, start, w);
  } else if (l==')') {
    addPixelsToBuffer (letterRPAREN, start, w);
  } else if (l==' ') {
    addPixelsToBuffer (letterSPACE, start, w);
  }
}

byte charWidth (char l) {
  if (l=='a') {
    return 4;
  } else if (l=='A') {
    return 5;
  } else if (l=='b') {
    return 4;
  } else if (l=='c') {
    return 4;
  } else if (l=='C') {
    return 4;
  } else if (l=='d') {
    return 4;
  } else if (l=='e') {
    return 4;
  } else if (l=='E') {
    return 5;
  } else if (l=='f') {
    return 4;
  } else if (l=='g') {
    return 4;
  } else if (l=='h') {
    return 3;
  } else if (l=='i') {
    return 3;
  } else if (l=='I') {
    return 3;
  } else if (l=='j') {
    return 2;
  } else if (l=='k') {
    return 4;
  } else if (l=='l') {
    return 3;
  } else if (l=='m') {
    return 5;
  } else if (l=='M') {
    return 5;
  } else if (l=='n') {
    return 4;
  } else if (l=='N') {
    return 4;
  } else if (l=='o') {
    return 4;
  } else if (l=='O') {
    return 5;
  } else if (l=='P') {
    return 4;
  } else if (l=='Q') {
    return 4;
  } else if (l=='p') {
    return 4;
  } else if (l=='q') {
    return 4;
  } else if (l=='r') {
    return 3;
  } else if (l=='s') {
    return 3;
  } else if (l=='t') {
    return 4;
  } else if (l=='T') {
    return 3;
  } else if (l=='u') {
    return 5;
  } else if (l=='U') {
    return 5;
  } else if (l=='V') {
    return 5;
  } else if (l=='v') {
    return 5;
  } else if (l=='w') {
    return 5;
  } else if (l=='x') {
    return 4;
  } else if (l=='y') {
    return 4;
  } else if (l=='Y') {
    return 4;
  } else if (l=='z') {
    return 4;
  } else if (l=='_') {
    return 4;
  } else if (l=='1') {
    return 3;
  } else if (l=='2') {
    return 4;
  } else if (l=='3') {
    return 4;
  } else if (l=='4') {
    return 5;
  } else if (l=='5') {
    return 4;
  } else if (l=='6') {
    return 4;
  } else if (l=='7') {
    return 4;
  } else if (l=='8') {
    return 4;
  } else if (l=='9') {
    return 4;
  } else if (l=='0') {
    return 4;
  } else if (l=='-') {
    return 3;
  } else if (l=='+') {
    return 3;
  } else if (l=='/') {
    return 3;
  } else if (l=='*') {
    return 3;
  } else if (l=='=') {
    return 3;
  } else if (l==':') {
    return 1;
  } else if (l=='.') {
    return 1;
  } else if (l==',') {
    return 2;
  } else if (l==';') {
    return 2;
  } else if (l=='|') {
    return 1;
  } else if (l=='!') {
    return 1;
  } else if (l=='?') {
    return 4;
  } else if (l=='(') {
    return 2;
  } else if (l==')') {
    return 2;
  } else if (l==' ') {
    return 2;
  } else {
    return 0;
  }
}
void initMotors () {
  pinMode(vibrator_pin,OUTPUT);
}

void enableVibrator () {
  digitalWrite(vibrator_pin,HIGH);
}

void disableVibrator () {
  digitalWrite(vibrator_pin,LOW);
}

void startVibrate (int time) {
  is_vibrating = true;
  vibrate_length = time;
  last_vibrate = millis();
  doVibrate();
}

void doVibrate () {
  if (is_vibrating && (millis()-last_vibrate < vibrate_length)) {
    enableVibrator();
  } else {
    last_vibrate = millis();
    is_vibrating = false;
    disableVibrator();
  }
}


#include <Wire.h>

//AD7746 definitions
#define I2C_ADDRESS  0x48 //0x90 shift one to the right

#define REGISTER_STATUS 0x00
#define REGISTER_CAP_DATA 0x01
#define REGISTER_VT_DATA 0x04
#define REGISTER_CAP_SETUP 0x07
#define REGISTER_VT_SETUP 0x08
#define REGISTER_EXC_SETUP 0x09
#define REGISTER_CONFIGURATION 0x0A
#define REGISTER_CAP_DAC_A 0x0B
#define REGISTER_CAP_DAC_B 0x0B
#define REGISTER_CAP_OFFSET 0x0D
#define REGISTER_CAP_GAIN 0x0F
#define REGISTER_VOLTAGE_GAIN 0x11

#define RESET_ADDRESS 0xBF

#define VALUE_UPPER_BOUND 16000000L 
#define VALUE_LOWER_BOUND 0xFL
#define MAX_OUT_OF_RANGE_COUNT 3
#define CALIBRATION_INCREASE 1
byte calibration;
byte outOfRangeCount = 0;

unsigned long offset = 0;

void initI2C()
{

  pinMode(capacitive_led_pin,OUTPUT);
  Wire.begin(); // sets up i2c for operation

  Serial.println("Initializing");

  Wire.beginTransmission(I2C_ADDRESS); // start i2c cycle
  Wire.send(RESET_ADDRESS); // reset the device
  Wire.endTransmission(); // ends i2c cycle

  //wait a tad for reboot
  delay(1);

  writeRegister(REGISTER_EXC_SETUP, _BV(3) | _BV(1) | _BV(0)); // EXC source A

  writeRegister(REGISTER_CAP_SETUP,_BV(7)); // cap setup reg - cap enabled

  Serial.println("Getting offset");
  offset = ((unsigned long)readInteger(REGISTER_CAP_OFFSET)) << 8;  
  Serial.print("Factory offset: ");
  Serial.println(offset);

  writeRegister(0x0A, _BV(7) | _BV(6) | _BV(5) | _BV(4) | _BV(3) | _BV(2) | _BV(0));  // set configuration to calib. mode, slow sample

  //wait for calibration
  delay(10);

  Serial.print("Calibrated offset: ");
  offset = ((unsigned long)readInteger(REGISTER_CAP_OFFSET)) << 8;  
  Serial.println(offset);

  writeRegister(REGISTER_CAP_SETUP,_BV(7)); // cap setup reg - cap enabled

  writeRegister(REGISTER_EXC_SETUP, _BV(3)); // EXC source A

  writeRegister(REGISTER_CONFIGURATION, _BV(7) | _BV(6) | _BV(5) | _BV(4) | _BV(3) | _BV(0)); // continuous mode

  calibrateI2C();

}

void checkI2C() // main program begins
{
  if (millis()>i2c_last_check+i2c_delay) {
    i2c_last_check = millis();
    
    i2c_value = readValue();
    
    /**
    Serial.print(offset);
    Serial.print("/");
    Serial.print((int)calibration);
    Serial.print("/");
    Serial.println(value);
    /**/
    
    if ((i2c_value<VALUE_LOWER_BOUND) or (i2c_value>VALUE_UPPER_BOUND)) {
      outOfRangeCount++;
    }
    if (0 && outOfRangeCount>MAX_OUT_OF_RANGE_COUNT) {
      if (i2c_value < VALUE_LOWER_BOUND) {
        calibrateI2C(-CALIBRATION_INCREASE);
      } 
      else {
        calibrateI2C(CALIBRATION_INCREASE);
      }
      outOfRangeCount=0;
    }
  
    //delay(50);
    // turn on LED if being touched
    if (i2c_value < capacitive_threshold) {
      Serial.print(offset);
      Serial.print("/");
      Serial.print((int)calibration);
      Serial.print("/");
      Serial.println(i2c_value);
      digitalWrite(capacitive_led_pin,HIGH);
    } else {
      digitalWrite(capacitive_led_pin,LOW);
    }
    /**/
  }
}

void calibrateI2C (byte direction) {
  calibration += direction;
  //assure that calibration is in 7 bit range
  calibration &=0x7f;
  writeRegister(REGISTER_CAP_DAC_A, _BV(7) | calibration);
}

void calibrateI2C() {
  calibration = 0;

  Serial.println("Calibrating CapDAC A");

  long value = readValue();

  while (value>VALUE_UPPER_BOUND && calibration < 128) {
    calibration++;
    writeRegister(REGISTER_CAP_DAC_A, _BV(7) | calibration);
    value = readValue();
  }
  Serial.println("done");
}

long readValue() {
  long ret = 0;
  uint8_t data[3];

  char status = 0;
  //wait until a conversion is done
  while (!(status & (_BV(0) | _BV(2)))) {
    //wait for the next conversion
    status = readRegister(REGISTER_STATUS);
  }

  unsigned long value =  readLong(REGISTER_CAP_DATA);

  value >>=8;
  //we have read one byte to much, now we have to get rid of it
  ret =  value;

  return ret;
}

void writeRegister(unsigned char r, unsigned char v)
{
  Wire.beginTransmission(I2C_ADDRESS);
  Wire.send(r);
  Wire.send(v);
  Wire.endTransmission();
}

void writeInteger(unsigned char r, unsigned int v) {
  writeRegister(r,(unsigned byte)v);
  writeRegister(r+1,(unsigned byte)(v>>8));
}

unsigned char readRegister(unsigned char r)
{
  unsigned char v;
  Wire.beginTransmission(I2C_ADDRESS);
  Wire.send(r);  // register to read
  Wire.endTransmission();

  Wire.requestFrom(I2C_ADDRESS, 1); // read a byte
  /**/
  while(Wire.available()==0) {
    // waiting
  }
  v = Wire.receive();
  /**/
  /**
  if (Wire.available()>0) {
    v = Wire.receive();
  }
  /**/
  return v;
}

void readRegisters(unsigned char r, unsigned int numberOfBytes, unsigned char buffer[])
{
  unsigned char v;
  Wire.beginTransmission(I2C_ADDRESS);
  Wire.send(r);  // register to read
  Wire.endTransmission();

  Wire.requestFrom(I2C_ADDRESS, numberOfBytes); // read a byte
  char i = 0;
  while (i<numberOfBytes) {
    while(!Wire.available()) {
      // waiting
    }
    buffer[i] = Wire.receive();
    i++;
  }
}

unsigned int readInteger(unsigned char r) {
  union {
    char data[2];
    unsigned int value;
  } 
  byteMappedInt;

  byteMappedInt.value = 0;

  Wire.beginTransmission(I2C_ADDRESS); // begin read cycle
  Wire.send(0); //pointer to first cap data register
  Wire.endTransmission(); // end cycle
  //after this, the address pointer is set to 0 - since a stop condition has been sent

  Wire.requestFrom(I2C_ADDRESS,r+2); // reads 2 bytes plus all bytes before the register

    while (!Wire.available()==r+2) {
      ; //wait
    }

  for (int i=r+1; i>=0; i--) {
    uint8_t c = Wire.receive();
    if (i<2) {
      byteMappedInt.data[i]= c;
    }
  }

  return byteMappedInt.value;

}

unsigned long readLong(unsigned char r) {
  union {
    char data[4];
    unsigned long value;
  } 
  byteMappedLong;

  byteMappedLong.value = 0L;

  Wire.beginTransmission(I2C_ADDRESS); // begin read cycle
  Wire.send(0); //pointer to first data register
  Wire.endTransmission(); // end cycle
  //the data pointer is reset anyway - so read from 0 on

  Wire.requestFrom(I2C_ADDRESS,r+4); // reads 2 bytes plus all bytes before the register

    while (!Wire.available()==r+4) {
      ; //wait
    }
  for (int i=r+3; i>=0; i--) {
    uint8_t c = Wire.receive();
    if (i<4) {
      byteMappedLong.data[i]= c;
    }
  }

  return byteMappedLong.value;

}
void behave () {
  checkI2C();
  BTcheck();
  if (digitalRead(capacitive_led_pin) && state!=7) {
    // was not being touched and now is
    state_changed = true;
    tmp_state = 7;
  } else if ((!digitalRead(capacitive_led_pin) && state==7) && tmp_state==0) {
    // not touched and no BT
    state_changed = true;
    tmp_state = 0;
  } else if (tmp_state!=0) {
    // BT data!
    state_changed = true;
  }
  
  if (state_changed) {
    Serial.println("clearing");
    Serial.print("new state: ");
    Serial.print(tmp_state);
    Serial.print(" msg: ");
    Serial.println(username);
    // there is a new state and we are done drawing
    username.toCharArray(my_message,username.length());
    ml = username.length();
    state = tmp_state;
    falseFlags();
    clearAll();
    switch (state) {
      case 0:
        beNothing();
        break;
      case 1:
        beTweet();
        break;
      case 2:
        beFollow();
        break;
      case 3:
        beMention();
        break;
      case 4:
        beMessage();
        break;
      case 5:
        bePhoto();
        break;
      case 6:
        beVideo();
        break;
      case 7:
        doSleep();
        break;
    }
  }
  drawMatrix();
  applyActions();
}

/**
  high-level behaviors
  0 = basic
  1 = tweet
  2 = follow
  3 = mention
  4 = message
  5 = photo
  6 = video
  7 = sleep
/**/

// 0
void beNothing () {
  String tmp = " please hug me ";
  tmp.toCharArray(my_message,tmp.length());
  ml = tmp.length();
}

// 1
void beTweet () {
  startBeep(500);
}

// 2
void beFollow () {
  startVibrate(1000);
}

// 3
void beMention () {
  startShining();
}

// 4
void beMessage () {
  startRead();
}

// 5
void bePhoto () {
  startVibrate(1000);
  startBeep(500);
}

// 6
void beVideo () {
  startVibrate(2000);
  startBeep(2000);
}

// 7
void doSleep () {
  if (!is_sleeping) {
    is_sleeping = true;
    start_sleeping = millis();
    String tmp = " zzzzzzzz ";
    tmp.toCharArray(my_message,tmp.length());
    ml = tmp.length();
  }
  int value;
  if (is_sleeping) {
    if (sleep_direction==1) {
      value = round((millis()-start_sleeping)*1023/sleep_length);
      if (value>1023) {
        start_sleeping = millis();
        value = 1023;
        sleep_direction = 2;
      }
    } else if (sleep_direction==2) {
      value = 1023 - round((millis()-start_sleeping)*1023/sleep_length);
      if (value<=0) {
        start_sleeping = millis();
        value = 0;
        sleep_direction = 1;
      }
    }
    analogWrite(a_led_pin,value);
    analogWrite(b_led_pin,value);
    analogWrite(c_led_pin,value);
  }
}

/**
low level behaviors
/**/

void applyActions () {
  doBeep();
  doVibrate();
  doShine();
  doRead();
}

void clearAll () {
  falseFlags();
  clearLEDs();
  resetScreen();
  disableVibrator();
  disableBuzzer();
}

void falseFlags() {
  state_changed = false;
  is_sleeping = false;
  is_vibrating = false;
  is_beeping = false;
  is_reading = false;
  is_shining = false;
}
void setup () {
  Serial.begin(9600);
  Serial1.begin(9600);
  // status led
  pinMode(status_led_pin, OUTPUT);
  setStatus("off");
  // crear la matriz con los pines
  initI2C();
  initMatrix();
  initLEDs();
  initBuzzer();
  refreshBuffer();
}

void loop () {
  behave();
}

void resetScreen () {
  // clean text part
  resetText();
  // start
  refreshBuffer();
  // change to bitmap mode
  mode = 1;
  last_bmp = millis();
}

void resetText () {
  current = 0;
  bc = 0;
  cc = 0;
  end_message = false;
}

void parseMessage () {
  String txt;
  // get the number part
  txt = serial_message.substring(0,1);
  char txtnum = txt[0];
  if (txtnum=='1') {
    tmp_state = 1;
  } else if (txtnum=='2') {
    tmp_state = 2;
  } else if (txtnum=='3') {
    tmp_state = 3;
  } else if (txtnum=='4') {
    tmp_state = 4;
  } else if (txtnum=='5') {
    tmp_state = 5;
  } else if (txtnum=='6') {
    tmp_state = 6;
  }
  // get the username
  txt = serial_message.substring(2,bytes_to_read-1);
  username = txt.replace('/',' ').trim();
  // add some space before and after
  username = "  " + username + "  ";
  serial_message = "";
  Serial.print("message parsed: ");
  Serial.print(tmp_state);
  Serial.print(" user: ");
  Serial.println(username);
}

void BTcheck () {
  // serial check
  if (Serial1.available() > 0){
      Serial.print("there is data: ");
      serial_message = "";
      setStatus("on");
      bytes_read = 0;
      while (bytes_read < bytes_to_read) {
        // second check
        if (Serial1.available() > 0) {
          // read data from serial port
          serial_data = Serial1.read();
          /**/
          if (serial_data == '_') {
            // if header or stop bytes before the limit digit reading
            break;
          }
          /**/
          serial_message += serial_data;
          bytes_read++;
        }
      }
      if (bytes_read != bytes_to_read-1) {
        // not enough bytes read but data in serial
        // kill it and go to base state
        Serial.flush();
        serial_message = "";
        username = "";
        tmp_state = 0;
        setStatus("off");
        bytes_read = 0;
      } else {
        parseMessage();
      }
  } else {
    Serial1.flush();
    setStatus("off");
    serial_message = "";
    username = "";
    tmp_state = 0;
    bytes_read = 0;
  }
}

void setStatus (String status) {
  if (status=="on") {
    digitalWrite(status_led_pin,HIGH);
  } else {
    digitalWrite(status_led_pin,LOW);
  }
}



